import os
import sys


def manage(settings_module, name=None, sys_argv=None, error_msg=None):
    """Invokes usual manage.py activity - launches django.\n
    Params:
    - settings_module - relative path to settings file, separated with dots.
    - name (optional) - the __name__ attribute of manage.py. If provided,
    will perform 'if __name__ == '__main__'' check.
    - sys_argv (optional) - if provided, will be used instead of sys.argv.
    """
    if name == '__main__' or not name:
        os.environ.setdefault('DJANGO_SETTINGS_MODULE', settings_module)
        try:
            from django.core.management import execute_from_command_line
        except ImportError as exc:
            raise ImportError(
                error_msg or
                "Couldn't import Django. Are you sure it's installed and "
                "available on your PYTHONPATH environment variable? Did you "
                "forget to activate a virtual environment?"
            ) from exc
        execute_from_command_line(sys_argv if sys_argv else sys.argv)


def meta(single=None, plural=None, display=None):
    """A compact way to set verbose name and the __str__ method in a model.\n
    WARNING! All meta defined this way won't register in migrations.\n
    Params:
    - single - if provided, sets verbose_name of the model.
    - plural - if provided, sets verbose_name_plural of the model.
    - display - if provided as a string, will be used as an attribute name
    to return with __str__.

    Example:

        @meta(display='name')
        class MyModel:
            name = CharField()

    translates to

        class MyModel:
            name = CharField()
            def __str__(self):
                return self.name

    You can also pass your own functions to display, in this case they will
    be set as __str__. Don't forget to pass `self` to your function first!
    """
    def decorator(Model):
        if single:
            Model._meta.verbose_name = single
        if single or plural:
            Model._meta.verbose_name_plural = plural if plural else single
        if type(display) == str:
            Model.__str__ = lambda self: self.__dict__[display]
        elif display:
            Model.__str__ = display
        return Model
    return decorator


def app_config(verbose_name=None, appname=None):
    """A compact way to set contrib for your app.\n
    Params:
    - verbose_name - if provided, sets verbose_name of the app. Otherwise,
    the normal name is used.
    - appname - if provided, sets an explicit name for the app. Otherwise,
    the folder name is used.

    Return value is a string that you should set your `default_app_config` to.
    """
    from django.apps import AppConfig
    import inspect
    vn = verbose_name
    f = inspect.getouterframes(inspect.currentframe(), 2)[1].filename
    path = os.path.basename(os.path.dirname(f))
    class AppConfig(AppConfig):
        name = appname or path
        verbose_name = vn or name
    setattr(sys.modules[path], 'AppConfig', AppConfig)
    return path + '.AppConfig'


def set_wsgi(settings_module=None, wsgi_module=None, name=None):
    """A compact way to set your wsgi.\n
    Params:
    - settings_module - if provided, performs
    `os.environ.setdefault('DJANGO_SETTINGS_MODULE', settings_module)`.
    - wsgi_module and name - if both are provided, performs a
    `name == wsgi_module` check before returning wsgi application.
    Name is supposed to be your file's __name__ in this situation.

    Return value is get_wsgi_application().
    """
    from django.core.wsgi import get_wsgi_application
    if settings_module:
        os.environ.setdefault('DJANGO_SETTINGS_MODULE', settings_module)
    if name == wsgi_module or not wsgi_module:
        return get_wsgi_application()


def log(settings, name_var=None, env_var=None, key_msg=False, prt_msg=True):
    """A short method for printing out some useful info.\n
    Params:
    - settings - your settings module.
    - name_var (str) - if provided, prints value of setting with this name.
    - env_var (str) - if provided, will print out stage info from setting
    with this name. If stage is `None`, will print `Debug`.
    - prt_msg - if `True`, prints and returns the output, else
    just returns it.
    """
    out = []
    if name_var:
        out.append(getattr(settings, name_var))
    if env_var:
        stage = (getattr(settings, env_var) or 'debug').upper()
        out.append(stage + ' stage.')
    if key_msg and os.getenv('SECRET_KEY'):
        out.append('\nFound a secret key!')
    msg = ' '.join(out)
    if prt_msg:
        print(msg)
    return msg


def load_app(module_name, objects=None):
    """Simply loads and returns an app module for you.\n
    Params:
    - module_name - module name of your app.
    - objects (str) - if provided, returns a certain part of the module
    instead of the whole thing. Example:
    `load_app('my_app', objects='views')` will return `my_app.views`.
    """
    from importlib import import_module
    if objects:
        return import_module(module_name + '.' + objects)
    else:
        return import_module(module_name)



from .djgo import manage, meta, app_config, set_wsgi, log, load_app
from . import admin
from django.conf import settings


def start_django(settings_module, name=None, sys_argv=None, error_msg=None):
    """Same as djgo.manage(). Invokes usual manage.py activity.\n
    Params:
    - settings_module - relative path to settings file, separated with dots.
    - name (optional) - the __name__ attribute of manage.py. If provided,
    will perform 'if __name__ == '__main__'' check.
    - sys_argv (optional) - if provided, will be used instead of sys.argv.
    """
    manage(settings_module, name, sys_argv, error_msg)



from django.contrib import admin


def quick_inline(model, verbose_name=None, fields=None, readonly_fields=None,
                can_delete=None, max_num=None, extra=None):
    """A shortcut for setting some tabular inline params.
    All except `model` are optional.
    """
    def decorator(Inline):
        Inline.model = model
        if fields:
            Inline.fields = fields
        if readonly_fields:
            Inline.readonly_fields = readonly_fields
        if verbose_name:
            Inline.verbose_name = verbose_name
        if can_delete:
            Inline.can_delete = can_delete
        if max_num:
            Inline.max_num = max_num
        if extra:
            Inline.extra = extra
        return Inline
    return decorator


def clear_models(*models, groups=False, users=False):
    """A shortcut for clearing models from admin site registry.\n
    Params:
    - models (iterable) - if provided, unregisters these models.
    - groups - if `True`, unregisters Group from `django.contrib.auth.models`.
    - users - if `True`, unregisters User from `django.contrib.auth.models`.
    """
    from django.contrib.auth.models import Group, User
    if users:
        admin.site.unregister(User)
    if groups:
        admin.site.unregister(Group)
    if models:
        for model in models:
            admin.site.unregister(model)


def urls(url='admin/'):
    """A shortcut for `path(url, admin.site.urls)`.
    """
    from django.urls import path
    if url[-1] != '/':
        url = url + '/'
    return path(url, admin.site.urls)


def meta(header=None, title=None):
    """A shortcut for setting `admin.site.site_header`
    and `admin.site.site_title`.
    """
    if header:
        admin.site.site_header = header
    if title:
        admin.site.site_title = title



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['djgo']

package_data = \
{'': ['*']}

install_requires = \
['django>=3.0.0,<4.0.0']

setup_kwargs = {
    'name': 'djgo',
    'version': '0.0.2',
    'description': 'A pack of shortcuts for Django.',
    'long_description': "# djgo\n\n## Description\n\nThis package provides some shortcuts for your django app.\n\n## Installation\n\n    pip install djgo\n\n- [PyPI page](https://pypi.org/project/djgo/)\n- [GitHub page](https://github.com/Ceterai/djgo/)\n\n## Usage\n\ndjgo has shortcuts for different parts of the app.\nTo use it, simply import it with `import djgo`.\n\n### manage.py\n\nMethod `start_django()` (or `manage()`) is a shortcut for launching Django.\n\n### wsgi.py\n\nMethod `set_wsgi()` is a shortcut that sets Django settings env variable and returns default wsgi application.\n\n### urls.py\n\nMethod `load_app()` is a shortcut that loads and returns an app module or its part.\n\n### app/contrib.py\n\nMethod `app_config()` is a shortcut that sets Django app config. Example:\n\n    from djpp import app_config\n    default_app_config = app_config(verbose_name='My Cool App')\n\n### app/models.py\n\nDecorator `@meta` is a shortcut that sets meta and \\_\\_str\\_\\_() for your model.  \nWARNING: All meta defined this way won't register in migrations.  \nExample:\n\n    @meta(single='My Client', plural='My Clients', display='name')\n    class Client(models.Model):\n        name = models.CharField()\n\nis equal to\n\n    class Client(models.Model):\n        name = models.CharField()\n        class Meta:\n            verbose_name = 'My Client'\n            verbose_name_plural = 'My Clients'\n        def \\_\\_str\\_\\_():\n            return self.name\n\nexcept the meta (name and verbose_name) won't register in migrations.\n\nYou can also pass functions to `display`:\n\n    @meta('My Client', display=lambda self: 'Cool ' + str(self.name))\n    class Client(models.Model):\n        name = models.CharField()\n\nis equal to\n\n    class Client(models.Model):\n        name = models.CharField()\n        class Meta:\n            name = 'My Client'\n            verbose_name = 'My Client'\n        def \\_\\_str\\_\\_():\n            return 'Cool ' + str(self.name)\n\nexcept the meta (name and verbose_name) won't register in migrations.\n\n### djgo.admin\n\ndjgo has a small set of methods for admin site and admin.py file:\n\n- `admin.urls` is a shortcut for `path('url', admin.site.urls)`, except adds '/' in the end of your url string if it doesn't have one.\n- `admin.meta` is a shortcut for setting `admin.site.site_header` and `admin.site.site_title`.\n- `admin.clear_models` is a shortcut for clearing models from admin site registry.\n- `admin.quick_inline` is a **decorator** for setting some tabular inline params.\n\n### Other features\n\nYou can also access settings through djgo with `djgo.settings`!\n\nLastly, djgo provides a small `log()` method for printing some info.\n",
    'author': 'Ceterai',
    'author_email': 'ceterai@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Ceterai/djgo',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
